'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.Parallax = undefined;

var _extends = Object.assign || function (target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i]; for (var key in source) { if (Object.prototype.hasOwnProperty.call(source, key)) { target[key] = source[key]; } } } return target; };

var _createClass = function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; }();

var _react = require('react');

var _react2 = _interopRequireDefault(_react);

var _propTypes = require('prop-types');

var _propTypes2 = _interopRequireDefault(_propTypes);

var _reactJss = require('react-jss');

var _reactJss2 = _interopRequireDefault(_reactJss);

var _reactGetNotDeclaredProps = require('react-get-not-declared-props');

var _reactGetNotDeclaredProps2 = _interopRequireDefault(_reactGetNotDeclaredProps);

var _warning = require('warning');

var _warning2 = _interopRequireDefault(_warning);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _objectWithoutProperties(obj, keys) { var target = {}; for (var i in obj) { if (keys.indexOf(i) >= 0) continue; if (!Object.prototype.hasOwnProperty.call(obj, i)) continue; target[i] = obj[i]; } return target; }

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

function _possibleConstructorReturn(self, call) { if (!self) { throw new ReferenceError("this hasn't been initialised - super() hasn't been called"); } return call && (typeof call === "object" || typeof call === "function") ? call : self; }

function _inherits(subClass, superClass) { if (typeof superClass !== "function" && superClass !== null) { throw new TypeError("Super expression must either be null or a function, not " + typeof superClass); } subClass.prototype = Object.create(superClass && superClass.prototype, { constructor: { value: subClass, enumerable: false, writable: true, configurable: true } }); if (superClass) Object.setPrototypeOf ? Object.setPrototypeOf(subClass, superClass) : subClass.__proto__ = superClass; }

/**
 * A component to render a parallax effect.
 *
 * @class
 * @extends PureComponent
 */
var Parallax = exports.Parallax = function (_PureComponent) {
  _inherits(Parallax, _PureComponent);

  function Parallax() {
    var _ref;

    var _temp, _this, _ret;

    _classCallCheck(this, Parallax);

    for (var _len = arguments.length, args = Array(_len), _key = 0; _key < _len; _key++) {
      args[_key] = arguments[_key];
    }

    return _ret = (_temp = (_this = _possibleConstructorReturn(this, (_ref = Parallax.__proto__ || Object.getPrototypeOf(Parallax)).call.apply(_ref, [this].concat(args))), _this), _this.imageIsLoaded = false, _this.handleResize = function () {
      _this.computeValues();

      _this.positionImage();
    }, _this.handleScroll = function () {
      _this.positionImage();
    }, _this.handleImageLoad = function () {
      _this.imageIsLoaded = true;

      _this.computeValues();

      _this.image.style.transform = 'translate3D(0, ' + -_this.overflowImageHeight + 'px, 0)';

      _this.positionImage();
    }, _temp), _possibleConstructorReturn(_this, _ret);
  }

  _createClass(Parallax, [{
    key: 'componentDidMount',


    /**
     * Add the event listener for when the user scrolls.
     */
    value: function componentDidMount() {
      window.addEventListener('scroll', this.handleScroll);
      window.addEventListener('resize', this.handleResize);
    }

    /**
     * Warn against changing the img prop.
     */

  }, {
    key: 'componentWillReceiveProps',
    value: function componentWillReceiveProps(nextProps) {
      (0, _warning2.default)(nextProps.img === this.props.img, 'You should not change the img prop of the Parallax');
    }

    /**
     * Remove the event listener again.
     */

  }, {
    key: 'componentWillUnmount',
    value: function componentWillUnmount() {
      window.removeEventListener('scroll', this.handleScroll);
      window.removeEventListener('resize', this.handleResize);
    }
  }, {
    key: 'computeValues',


    /**
     * Compute some static values which don't change when the user scrolls.
     *
     * @private
     */
    value: function computeValues() {
      this.rootHeight = this.root.getBoundingClientRect().height;
      this.imageHeight = this.image.getBoundingClientRect().height;
      this.pixelsToScroll = window.innerHeight - this.rootHeight;
      this.overflowImageHeight = this.imageHeight - this.rootHeight;
    }

    /**
     * Reposition the image based on the current scroll position.
     *
     * @private
     */

  }, {
    key: 'positionImage',
    value: function positionImage() {
      if (!this.imageIsLoaded) {
        return;
      }

      var _root$getBoundingClie = this.root.getBoundingClientRect(),
          top = _root$getBoundingClie.top,
          bottom = _root$getBoundingClie.bottom;

      var _window = window,
          innerHeight = _window.innerHeight;

      // Check if the parallax is completely visible

      if (top >= 0 && bottom <= innerHeight) {
        var scrollPos = 1 - Math.abs((innerHeight - this.rootHeight - top) / this.pixelsToScroll);
        var transform = scrollPos * this.overflowImageHeight;

        this.image.style.transform = 'translate3D(0, ' + -transform + 'px, 0)';
      }
    }

    /**
     * Recompute the static values and reposition the image.
     *
     * @private
     */


    /**
     * Update the image position on a scroll event.
     * Only update it when the root element is completely visible.
     *
     * @private
     */


    /**
     * When the image loads, compute the values and initially position the image.
     */

  }, {
    key: 'render',
    value: function render() {
      var _this2 = this;

      var _props = this.props,
          classes = _props.classes,
          img = _props.img,
          className = _props.className,
          children = _props.children,
          props = _objectWithoutProperties(_props, ['classes', 'img', 'className', 'children']);

      return _react2.default.createElement(
        'div',
        _extends({
          role: 'presentation',
          className: classes.root + ' ' + className,
          ref: function ref(element) {
            _this2.root = element;
          }
        }, (0, _reactGetNotDeclaredProps2.default)(props, Parallax, 'sheet')),
        _react2.default.createElement('img', {
          width: '100%',
          src: img,
          className: classes.image,
          alt: 'parallax',
          ref: function ref(element) {
            _this2.image = element;
          },
          onLoad: this.handleImageLoad
        }),
        _react2.default.createElement(
          'div',
          { className: classes.content },
          children
        )
      );
    }
  }]);

  return Parallax;
}(_react.PureComponent);

Parallax.propTypes = {
  classes: _propTypes2.default.shape({
    root: _propTypes2.default.string.isRequired,
    image: _propTypes2.default.string.isRequired,
    content: _propTypes2.default.string.isRequired
  }).isRequired,
  img: _propTypes2.default.string.isRequired,
  children: _propTypes2.default.node,
  className: _propTypes2.default.string
};
Parallax.defaultProps = {
  children: '',
  className: ''
};
Parallax.styles = {
  root: {
    composes: 'parallax',
    position: 'relative',
    overflow: 'hidden'
  },

  image: {
    composes: 'parallax--image',
    position: 'absolute',
    top: 0,
    right: 0,
    bottom: 'auto',
    left: 0,
    transition: 'transform 8ms linear',
    zIndex: 0,
    willChange: 'transform'
  },

  content: {
    composes: 'parallax--content',
    zIndex: 1
  }
};
exports.default = (0, _reactJss2.default)(Parallax.styles)(Parallax);